"use strict";

const {Menu, app} = require("electron");
const path        = require("path");

class MainMenu {
	constructor(parentApp) {
		this.app  = parentApp;
		this.menu = null;
		this.refresh();
	}

	refresh() {
		this.buildItems();
		Menu.setApplicationMenu(this.menu);
	}

	buildItems() {
		const settingsItem = {
			label : "Settings",
			click : () => this.app.windowManager.windows.settings.show()
		};
		const template     = [
			...(process.platform === "darwin" ? [{
				label: app.name,
				submenu: [
					{ role: "about" },
					{...settingsItem},
					{ type: "separator" },
					{ role: "services" },
					{ type: "separator" },
					{ role: "hide" },
					{ role: "hideOthers" },
					{ role: "unhide" },
					{ type: "separator" },
					{ role: "quit" }
				]
			}] : []),
			{
				label   : "File",
				submenu : [
					{
						label       : "Open",
						accelerator : "CommandOrControl+O",
						click       : () => this.app.openFile()
					},
					{
						id          : "close-file",
						label       : "Close",
						accelerator : "CommandOrControl+W",
						click       : () => this.app.closeFile(),
						enabled     : this.app.currentFile !== ""
					}, ...(process.platform !== "darwin" ? [{type : "separator"}, {...settingsItem}] : [])
				]
			},
			{
				role : "windowMenu"
			}
		];
		template[1].submenu.push({
			id      : "recent-files",
			label   : "Recent files",
			enabled : this.app.recentFiles.length > 0,
			...(this.app.recentFiles.length > 0 ? {submenu : this.app.recentFiles.map(item => ({
				label : path.basename(item),
				click : () => this.app.openFile("" + item)
			}))} : {}),
		});

		this.menu = Menu.buildFromTemplate(template);
	}

}

module.exports = MainMenu;