"use strict";

const {Lexer, marked} = require("marked");
const util            = require("util");

const tokenizer = {
	list : (source) => {
		console.log(source);

		return false;
	}
};

marked.use({tokenizer});


class Slide {
	title;
	content;

	constructor(content = []) {
		this.title   = false;
		this.content = content;
	}

	append(object) {
		this.content.push(object);
	}
}

class SlideDeck {
	metaData;
	slides;

	constructor() {
		this.metaData = null;
		this.slides   = [];
	}

	push(slide) {
		this.slides.push(slide);
	}

	unshift(slide) {
		this.slides.unshift(slide);
	}
}

const tokenize = string => {
	//console.log(util.inspect(marked.Lexer.rules.block.listItemStart, true, null, true));
	return new Lexer({gfm : true}).lex(string);
};

const injectTitle = (deck) => {
	const title = [];
	const meta  = deck.metaData;

	if (meta?.icon)
		title.push({
			type   : "image",
			href   : meta.icon
		});
	if (meta?.title)
		title.push({
			type   : "heading",
			level  : 1,
			tokens : [
				{
					type : "text",
					text : meta.title
				}
			]
		});
	if (meta?.subtitle)
		title.push({
			type   : "heading",
			level  : 2,
			tokens : [
				{
					type : "text",
					text : meta.subtitle
				}
			]
		});
	if (meta?.author)
		title.push({
			type   : "paragraph",
			tokens : [
				{
					type : "text",
					text : `–  ${meta.author} –`
				},
			]
		});
	if (meta?.email)
		title.push({
			type   : "paragraph",
			tokens : [
				{
					type : "text",
					text : `<${meta.email}>`
				},
			]
		});

	if (title.length > 0) {
		const slide = new Slide(title);
		slide.title = true;
		deck.unshift(slide);
	}

	return deck;
}

const parseMetaData = source => {
	const meta       = {};

	for (const attribute of source.split('\n')) {
		const trimmed = attribute.trim();
		if (trimmed.length < 1 || (trimmed.charAt(0) === '/' && trimmed.charAt(1) === '/'))
			continue;

		const [key, value] = trimmed.split(':');
		if (key.trim().length < 1 || value.trim().length < 1)
			continue;
		meta[key.trim()]   = value.trim();
	}

	console.table(meta);

	return meta;
};

const parser = string => {
	const tokenStream  = tokenize(string);
	const slideDeck    = new SlideDeck();
	let   currentSlide = new Slide();

	for (const token of tokenStream) {
		if (token.type === "space" || (token.type === "heading" && token.depth === 1) || token.type === "html") {
			if (currentSlide.content.length > 0) {
				slideDeck.push(currentSlide);
				currentSlide = new Slide();
			}
			// basically, if this is the forst comment, we expect meta data
			if (!slideDeck.metaData && token.type === "html" && token.text.charAt(2) === '-') {
				const metaString = token.text.replace(/(<!--|-->)/gi, "");
				slideDeck.metaData = parseMetaData(metaString);
			}

			if (token.type === "heading")
				slideDeck.push(new Slide([token]));
			continue;
		}
		currentSlide.append(token);
	}
	if (currentSlide.content.length > 0)
		slideDeck.push(currentSlide);

	return injectTitle(slideDeck);
};

module.exports = {
	parser
}