"use strict";

const {ipcMain, app, dialog} = require("electron");
const fsn                    = require("fs");
const fs                     = require("fs/promises")
const path                   = require("path");

class SettingsManager {
	app;
	data;

	constructor(app) {
		this.app  = app;
		this.data = null;

		if (!SettingsManager.CheckFileSystem()) {
			if (!fsn.existsSync(SettingsManager.Folder))
				fsn.mkdirSync(SettingsManager.Folder);
			fsn.writeFileSync(SettingsManager.File, "{}");
		}
		this.data = JSON.parse(fsn.readFileSync(SettingsManager.File, {encoding : "utf-8"}));

		ipcMain.handle("SettingsManager::get",    (_, key, defaultValue = null) => this.get(key, defaultValue));
		ipcMain.handle("SettingsManager::set",    (_, key, value) => this.set(key, value));
		ipcMain.handle("SettingsManager::all",    () => this.data);
		ipcMain.on("SettingsManager::reset",      () => this.reset());
	}

	get(key, defaultValue = null) {
		if (Object.keys(this.data).includes(key))
			return this.data[key];
		return defaultValue;
	}

	set(key, value) {
		this.data[key] = value;
		this.save();
	}

	change() {
		this.app.windowManager.mainWindow?.send("SettingsManager::change", this.data);
	}

	save() {
		fs.writeFile(SettingsManager.File, JSON.stringify(this.data, null, 4));
		this.change();
	}

	async reset() {
		const {response} = await dialog.showMessageBox(this.app.windowManager.windows.settings, {
			title     : "Reset all settings",
			message   : "Do you really want to revert ALL settings back to default?",
			buttons   : ["yes", "no"],
			defaultId : 0
		});
		if (response === 1)
			return;
		this.data = {};
		this.save();
	}

	static CheckFileSystem() {

		return fsn.existsSync(SettingsManager.Folder) && fsn.existsSync(SettingsManager.File);
	}
}
SettingsManager.Folder = app.getPath("userData");
SettingsManager.File   = path.join(SettingsManager.Folder, "config.json");

module.exports = SettingsManager;