import React, {useEffect, useState, useContext, useCallback} from "react";

import SlidesList      from "./SlidesList";
import Mode            from "../models/Mode";
import Slide           from "./Slide";
import KeyboardControl from "./KeyboardControl";
import Blackout        from "./Blackout";
import NoFile          from "./NoFile";
import Toolbar         from "./Toolbar";
import Tips            from "./Tips";
import Editor          from "./Editor";

import SlideContext    from "../shared/SlideContext";
import SettingsContext from "../shared/SettingsContext";

const Ation = () => {
	const {font, highlightColor, backgroundColor, color} = useContext(SettingsContext);
	const [mode, setMode]                                = useState(Mode.NORMAL);
	const [deck, setDeck]                                = useState([]);
	const [meta, setMeta]                                = useState(null);
	const [slide, setSlide]                              = useState(0);
	const [basePath, setBasePath]                        = useState("");
	const [showTips, setShowTips]                        = useState(false);
	const [version, setVersion]                          = useState("0.0.0");
	const [source, setSource]                            = useState("");

	useEffect(() => {
		window.api.onFileOpen(presentation => {
			window.api.clearCache();
			const [newBasePath, slideDeck, fileContents] = presentation;
			if (!slideDeck)
				return;
			if (slide >= slideDeck.length)
				setSlide(0);
			setSource(fileContents);
			setMeta(slideDeck.metaData);
			setBasePath(newBasePath);
			setDeck(slideDeck.slides);
		});
		window.api.onFileClose(() => {
			setBasePath("");
			setSlide(0);
			setDeck([]);
		});
		(async set => set(await window.api.appVersion()))(setVersion);
		//(async setFont => setFont(await window.appSettings.get("font", "Iosevka")))(setFont);

	}, [basePath, slide]);

	const openFile = () => {
		window.api.openFileDialog();
	};

	const toggleEdit = useCallback(() => {
		if (mode === Mode.NORMAL)
			setMode(Mode.EDIT);
		else if (mode === Mode.EDIT)
			setMode(Mode.NORMAL);
	}, [mode]);

	return (
		<>
			{deck.length < 1 ?
				<NoFile openFile={openFile} />
			: (
				<SlideContext.Provider value={{slide, setSlide, mode, setMode, basePath, slideCount : deck.length}}>
					<section className={`window${mode === Mode.PRESENT ? " fullscreen" : (mode === Mode.EDIT ? " edit" : "")}`}>
						<Toolbar openFile={openFile} setShowTips={setShowTips} version={version} toggleEdit={toggleEdit} />
						<SlidesList deck={deck} meta={meta} font={font} />
						<main className={`main ${mode === Mode.EDIT ? "edit" : ""}`} style={{
							"--color-hightlight" : meta?.color_highlight || highlightColor,
							"--color-slideBackground" : meta?.color_background || backgroundColor,
							"--color-slideForeground" : meta?.color_text || color
						}}>
							<Slide data={deck[slide] || null} style={{fontFamily : meta?.font || font}}/>
							<Editor show={mode === Mode.EDIT} source={source} toggleEdit={toggleEdit} />
						</main>
						<Tips show={showTips} />
					</section>
					<Blackout show={mode === Mode.BLACKOUT} />
					<KeyboardControl mode={mode} setMode={setMode} deck={deck} openFile={openFile} setShowTips={setShowTips} toggleEdit={toggleEdit} />
				</SlideContext.Provider>
			)}
		</>
	);
};

export default Ation;