import {useContext, useEffect, version} from "react";

import SlideContext from "../shared/SlideContext";
import Mode         from "../models/Mode";

const KeyboardControl = ({openFile, mode, setMode, deck, setShowTips, toggleEdit}) => {
	const {slide, setSlide} = useContext(SlideContext);

	useEffect(() => {
		const keyUpHandler = event => {
			switch(event.key) {
				case "Tab":
					setShowTips(false);
					break;
				default:
					return;
			}
		}

		const keyHandler = event => {
			const commandOrControl = event.metaKey || event.ctrlKey;

			switch(event.key) {
				case "Escape":
					setShowTips(false);
					break;
				default:
					break;
			}
			if (!mode || !setMode || !deck || !setShowTips)
				return;

			if (commandOrControl && event.key === "e")
				toggleEdit();

			if (mode === Mode.EDIT)
				return;

			switch (event.key) {
				case "F5":
					if (deck.length <= 0)
						return;
					setSlide(0);
					document.documentElement.requestFullscreen();
					setMode(Mode.PRESENT);
					break;
				case "ArrowLeft":
				case "j":
				case "PageUp": // PRESENTER
					if (slide <= 0)
						return;
					setSlide(slide - 1);
					break;
				case "ArrowRight":
				case "k":
				case "PageDown": // PRESENTER
					if (slide >= deck.length -1)
						return;
					setSlide(slide + 1);
					break;
				case "b": // PRESENTER
					if (mode === Mode.NORMAL || mode === Mode.EDIT)
						return;
				 	setMode(mode === Mode.BLACKOUT ? Mode.PRESENT : Mode.BLACKOUT);
					break;
				case "g":
					setSlide(0);
					break;
				case "G":
					setSlide(deck.length - 1);
					break;
				case "Tab":
					event.preventDefault();
					setShowTips(true);
					break;
				default:
					return;
			}
		};

		const fullscreenHandler = () => {
			if (!document.fullscreenElement)
				setMode(Mode.NORMAL);
		};

		window.addEventListener("keydown", keyHandler);
		window.addEventListener("keyup", keyUpHandler);
		window.addEventListener("fullscreenchange", fullscreenHandler);

		return () => {
			window.removeEventListener("keydown", keyHandler);
			window.removeEventListener("keyup", keyUpHandler);
			window.removeEventListener("fullscreenchange", fullscreenHandler);
		}

	}, [slide, setSlide, mode, setMode, deck, setShowTips, openFile, toggleEdit]);
};

export default KeyboardControl;