import React, {useState, useEffect} from "react";

const Appearance = ({fonts}) => {
	const [font, setFont]                       = useState("");
	const [highlightColor, setHighlightColor]   = useState("");
	const [backgroundColor, setBackgroundColor] = useState("");
	const [color, setColor]                     = useState("");

	const changeFont = event => {
		const value = event.target.value;
		setFont(value);
		window.appSettings.set("font", value);
	};

	const changeColor = event => {
		const value = event.target.value;
		setHighlightColor(value);
		window.appSettings.set("highlightColor", value);
	};

	const changeBackground = event => {
		const value = event.target.value;
		setBackgroundColor(value);
		window.appSettings.set("backgroundColor", value);
	};
	
	const changeForegroundColor = event => {
		const value = event.target.value;
		setColor(value);
		window.appSettings.set("color", value);
	};

	useEffect(() => {
		(async () => {
			setBackgroundColor(await window.appSettings.get("backgroundColor", "#1a1a1a"));
			setHighlightColor(await window.appSettings.get("highlightColor", "#e6c17b"));
			setFont(await window.appSettings.get("font", "Iosevka"));
			setColor(await window.appSettings.get("color", "#dddddd"));
		})(setFont, setHighlightColor, setBackgroundColor, setColor);
	}, []);

	return (
		<section className="grid">
			<label>Font in slides:</label>
			<select value={font} onChange={changeFont}>
				{fonts.map(font => font.trim().replace(/(^"|"$)/g, "")).map(font => <option key={font} value={font}>{font}</option>)}
			</select>
			<label>Global highlight color:</label>
			<input type="color" value={highlightColor} onChange={changeColor} />
			<label>Slide background color:</label>
			<input type="color" value={backgroundColor} onChange={changeBackground} />
			<label>Slide font color:</label>
			<input type="color" value={color} onChange={changeForegroundColor} />
		</section>
	);
};

export default Appearance;