import React, {useState} from "react";

const extension = filename => {
	const parts = filename.split('.');

	return parts[parts.length - 1];
};

const withDrop = Component => () => {
	const [drag, setDrag] = useState(false);

	const handleDragOver = event => {
		event.preventDefault();
		event.stopPropagation();
		setDrag(true);
	};

	const handleDragLeave = event => {
		event.preventDefault();
		event.stopPropagation();
		setDrag(false);
	}

	const handleDrop = event => {
		event.preventDefault();
		event.stopPropagation();
		const file = event.dataTransfer?.items[0].getAsFile();
		if (extension(file.name.toLowerCase()) === "md") {
			window.api.openFile(file.path);
		}
		setDrag(false);
	};

	return (
		<section onDragLeave={handleDragLeave} onDragOver={handleDragOver} onDrop={handleDrop} style={{opacity : drag ? 0.2 : 1}}>
			<Component />
		</section>
	);
};


export default withDrop;